"""add disp action

Peek Plugin Database Migration Script

Revision ID: 42a8dc25e588
Revises: f697025b2013
Create Date: 2020-05-10 15:20:02.102210

"""

# revision identifiers, used by Alembic.
revision = '42a8dc25e588'
down_revision = 'f697025b2013'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('DispBase', sa.Column('action', sa.SmallInteger(), nullable=True), schema='pl_diagram')
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('DispBase', 'action', schema='pl_diagram')
    # ### end Alembic commands ###


from typing import Optional, List

from vortex.Tuple import Tuple, addTupleType, TupleField

from peek_plugin_diagram._private.PluginNames import diagramTuplePrefix
from peek_plugin_diagram.tuples.model.ImportLiveDbDispLinkTuple import \
    ImportLiveDbDispLinkTuple
from peek_plugin_diagram.tuples.shapes.ImportDispPolylineTuple import \
    ImportDispPolylineTuple


@addTupleType
class ImportDispTextTuple(Tuple):
    """ Imported Display Text

    This tuple is used by other plugins to load TEXT objects into the diagram.

    """
    __tupleType__ = diagramTuplePrefix + 'ImportDispTextTuple'

    ### BEGIN DISP COMMON FIELDS ###

    # The actions to perform when this line is clicked
    ACTION_NONE = ImportDispPolylineTuple.ACTION_NONE
    ACTION_POSITION_ON = ImportDispPolylineTuple.ACTION_POSITION_ON
    # For ACTION_POSITION_ON, Add the following to the display data
    # data['actionPos'] = {k='coordSetKey', x=x, y=y, z=zoom}

    #: Key, This value is a unique ID of the object that this graphic represents
    # It's used to link this graphical object to objects in other plugins, like vertices
    # in the peek-plugin-graphdb plugin.
    # Length = 50
    key: str = TupleField()

    #: Selectable, Is is this item selectable?, the layer also needs selectable=true
    selectable: bool = TupleField()

    #: Overlay, Is is this shape an overlay?, Overlays are sometimes used to add dynamic
    # data to the diagram, such as a Job, Operation, or placing a green box over a red
    # one to change it's state.
    overlay: bool = TupleField()

    #: Action, An action to perform when this display item is clicked.
    # See the ACTION_NONE constants for values.
    action: Optional[int] = TupleField(None)

    #: Data, Generic data, this is passed to the popup context in the UI.
    # peek_plugin_diagram doesn't care as long as it's json compatible or None
    # Json length Length = 400
    data: Optional[dict] = TupleField(None)

    #: The hash of the level to link to (Matches ImportDispLevel.importHash)
    levelHash: str = TupleField()

    #: The hash of the layer to link to (Matches ImportDispLayer.importHash)
    layerHash: str = TupleField()

    #: The unique hash of this display object
    importHash: str = TupleField()

    #: The Z Order of this display object when compared against other objects on
    # same layer and level.
    zOrder: int = TupleField()

    #: The unique hash for all the display items imported in a group with this one.
    #: for example, a page or tile reference.
    importGroupHash: str = TupleField()

    #: The key of the ModelSet to import into
    modelSetKey: str = TupleField()

    #: The Key of the Coordinate Set to import into
    coordSetKey: str = TupleField()

    #: Related links to LiveDB values for this display item
    liveDbDispLinks: List[ImportLiveDbDispLinkTuple] = TupleField()

    #: Parent DispGroup Hash, If this disp is part of a disp group then set this field to
    # the ImportDispGroupTuple.importHash fields value
    # NOTE: If this disp is part of a display group, then the GEOM coordinates need to
    # be relative to 0x0.
    # NOTE: Disps that are apart of a group must all be imported with the same
    # importGroupHash, during the same import call.
    parentDispGroupHash: str = TupleField()

    ### BEGIN FIELDS FOR THIS DISP ###

    textStyleHash: str = TupleField()
    colorHash: Optional[str] = TupleField()

    H_ALLIGN_LEFT = -1
    H_ALLIGN_CENTER = 0
    H_ALLIGN_RIGHT = 1
    horizontalAlign: int = TupleField(H_ALLIGN_CENTER)

    V_ALLIGN_TOP = -1
    V_ALLIGN_CENTER = 0
    V_ALLIGN_BOTTOM = 1
    verticalAlign: int = TupleField(V_ALLIGN_CENTER)

    geom: List[float] = TupleField()

    #: The rotation of the text
    rotation: float = TupleField()

    #: The value of the text
    text: str = TupleField()

    #: This field stores text with format strings that are used to create the text above.
    textFormat: Optional[str] = TupleField()

    #: Text Height (Optional, defaults to height in TextStyle lookup)
    textHeight: Optional[float] = TupleField()

    #: Text Horizontal Stretch, The scale to stretch the text horizontally
    textHStretch: float = TupleField(1)



import logging
from typing import List

from peek_abstract_chunked_index.private.server.controller.ACIProcessorQueueControllerABC import \
    ACIProcessorQueueControllerABC, ACIProcessorQueueBlockItem
from peek_abstract_chunked_index.private.server.controller.ACIProcessorStatusNotifierABC import \
    ACIProcessorStatusNotifierABC
from peek_abstract_chunked_index.private.tuples.ACIProcessorQueueTupleABC import \
    ACIProcessorQueueTupleABC
from peek_plugin_diagram._private.server.controller.StatusController import \
    StatusController
from peek_plugin_diagram._private.storage.DispIndex import \
    DispIndexerQueue as DispIndexerQueueTable
from peek_plugin_diagram._private.storage.Display import DispBase, DispNull, DispGroup, \
    DispPolyline, DispGroupPointer, DispPolygon, DispEdgeTemplate, DispText, DispEllipse
from peek_plugin_diagram._private.storage.LiveDbDispLink import LiveDbDispLink
from vortex.DeferUtil import deferToThreadWrapWithLogger

logger = logging.getLogger(__name__)


class _Notifier(ACIProcessorStatusNotifierABC):
    def __init__(self, adminStatusController: StatusController):
        self._adminStatusController = adminStatusController

    def setProcessorStatus(self, state: bool, queueSize: int):
        self._adminStatusController.status.displayCompilerQueueStatus = state
        self._adminStatusController.status.displayCompilerQueueSize = queueSize
        self._adminStatusController.notify()

    def addToProcessorTotal(self, delta: int):
        self._adminStatusController.status.displayCompilerProcessedTotal += delta
        self._adminStatusController.notify()

    def setProcessorError(self, error: str):
        self._adminStatusController.status.displayCompilerLastError = error
        self._adminStatusController.notify()


class DispCompilerQueueController(ACIProcessorQueueControllerABC):
    QUEUE_ITEMS_PER_TASK = 500
    POLL_PERIOD_SECONDS = 0.200

    QUEUE_BLOCKS_MAX = 20
    QUEUE_BLOCKS_MIN = 4

    WORKER_TASK_TIMEOUT = 60.0

    DEDUPE_LOOK_AHEAD_MIN_ROWS = 10 * 6  # a million rows
    DEDUPE_PERIOD_SECONDS: float = 60.0

    _logger = logger
    _QueueDeclarative: ACIProcessorQueueTupleABC = DispIndexerQueueTable
    _VacuumDeclaratives = (DispIndexerQueueTable,
                           DispBase, DispNull, DispText, DispPolygon, DispPolyline,
                           DispEllipse, DispGroup, DispEdgeTemplate, DispGroupPointer,
                           LiveDbDispLink)

    def __init__(self, dbSessionCreator, statusController: StatusController):
        ACIProcessorQueueControllerABC.__init__(self, dbSessionCreator,
                                                _Notifier(statusController))

    def _sendToWorker(self, block: ACIProcessorQueueBlockItem):
        from peek_plugin_diagram._private.worker.tasks.DispCompilerTask import \
            compileDisps

        return compileDisps.delay(block.itemsEncodedPayload)

    def _processWorkerResults(self, results):
        pass

    # ---------------
    # Deduplicate methods

    def _dedupeQueueSql(self, lastFetchedId: int, dedupeLimit: int):
        return '''
                 with sq_raw as (
                    SELECT "id", "dispId"
                    FROM pl_diagram."DispCompilerQueue"
                    WHERE id > %(id)s
                    LIMIT %(limit)s
                ), sq as (
                    SELECT min(id) as "minId", "dispId"
                    FROM sq_raw
                    GROUP BY  "dispId"
                    HAVING count("dispId") > 1
                )
                DELETE
                FROM pl_diagram."DispCompilerQueue"
                     USING sq sq1
                WHERE pl_diagram."DispCompilerQueue"."id" != sq1."minId"
                    AND pl_diagram."DispCompilerQueue"."id" > %(id)s
                    AND pl_diagram."DispCompilerQueue"."dispId" = sq1."dispId"

            ''' % {'id': lastFetchedId, 'limit': dedupeLimit}

    # ---------------
    # Insert into Queue methods

    @deferToThreadWrapWithLogger(logger)
    def queueDisps(self, dispIds):
        return self.queueDispIdsToCompile(dispIds, self._dbSessionCreator)

    @classmethod
    def queueDispIdsToCompile(cls, dispIdsToCompile: List[int], dbSessionCreator):
        if not dispIdsToCompile:
            return

        ormSession = dbSessionCreator()
        try:
            cls.queueDispIdsToCompileWithSession(dispIdsToCompile, ormSession)
            ormSession.commit()

        finally:
            ormSession.close()

    @staticmethod
    def queueDispIdsToCompileWithSession(dispIdsToCompile: List[int], ormSessionOrConn):
        if not dispIdsToCompile:
            return

        logger.debug("Queueing %s disps for compile", len(dispIdsToCompile))

        inserts = []
        for dispId in dispIdsToCompile:
            inserts.append(dict(dispId=dispId))

        ormSessionOrConn.execute(DispIndexerQueueTable.__table__.insert(), inserts)



import logging

import ujson
from peek_plugin_diagram._private.server.controller.BranchLiveEditController import \
    BranchLiveEditController
from peek_plugin_diagram._private.storage.branch.BranchIndex import BranchIndex
from peek_plugin_diagram._private.tuples.branch.BranchKeyToIdMapTuple import \
    BranchKeyToIdMapTuple
from peek_plugin_diagram._private.tuples.branch.BranchLiveEditTuple import \
    BranchLiveEditTuple
from peek_plugin_diagram._private.tuples.branch.BranchTuple import BranchTuple
from peek_plugin_diagram._private.tuples.branch.BranchUpdateTupleAction import \
    BranchUpdateTupleAction
from peek_plugin_diagram._private.worker.tasks.branch.BranchIndexImporter import \
    createOrUpdateBranches
from peek_plugin_diagram._private.worker.tasks.branch.BranchIndexUpdater import \
    updateBranches
from peek_plugin_livedb.server.LiveDBWriteApiABC import LiveDBWriteApiABC
from twisted.internet import defer
from twisted.internet.defer import inlineCallbacks, Deferred
from vortex.DeferUtil import vortexLogAndConsumeFailure
from vortex.Payload import Payload
from vortex.TupleAction import TupleActionABC
from vortex.TupleSelector import TupleSelector
from vortex.handler.TupleActionProcessor import TupleActionProcessorDelegateABC
from vortex.handler.TupleDataObservableHandler import TupleDataObservableHandler

logger = logging.getLogger(__name__)


class BranchUpdateController(TupleActionProcessorDelegateABC):
    """ Branch Update Controller

    This controller handles the branch updates from the UI

    """

    def __init__(self, liveDbWriteApi: LiveDBWriteApiABC,
                 tupleObservable: TupleDataObservableHandler,
                 liveEditController: BranchLiveEditController,
                 dbSessionCreator):
        self._liveDbWriteApi = liveDbWriteApi
        self._tupleObservable = tupleObservable
        self._liveEditController = liveEditController
        self._dbSessionCreator = dbSessionCreator

    def shutdown(self):
        self._liveDbWriteApi = None
        self._tupleObservable = None
        self._liveEditController = None
        self._dbSessionCreator = None

    @inlineCallbacks
    def updateBranch(self, branchEncodedPayload: bytes):
        yield None
        raise NotImplemented("BranchUpdateController.updateBranch")

    @inlineCallbacks
    def importBranches(self, branchesEncodedPayload: bytes):
        yield createOrUpdateBranches.delay(branchesEncodedPayload)

        self._updateBranchKeyToIdMap()

    def processTupleAction(self, tupleAction: TupleActionABC) -> Deferred:
        if not isinstance(tupleAction, BranchUpdateTupleAction):
            raise Exception("Unhandled tuple action %s" % tupleAction)

        logger.debug("Received branch save for branch %s by %s",
                     tupleAction.branchTuple.key,
                     tupleAction.branchTuple.updatedByUser)

        d = self.__processUpdateFromClient(tupleAction)
        d.addErrback(vortexLogAndConsumeFailure, logger)
        return defer.succeed([])

    @inlineCallbacks
    def __processUpdateFromClient(self, tupleAction: TupleActionABC) -> Deferred:

        dbSession = self._dbSessionCreator()
        try:
            encodedPayload = yield Payload(tuples=[tupleAction.branchTuple]) \
                .toEncodedPayloadDefer()

            yield updateBranches.delay(tupleAction.modelSetId, encodedPayload)

            # Load the branch from the DB and tell the LiveDb update
            # that there is an update.
            branchIndex = dbSession.query(BranchIndex) \
                .filter(BranchIndex.coordSetId == tupleAction.branchTuple.coordSetId) \
                .filter(BranchIndex.key == tupleAction.branchTuple.key) \
                .one()

            branchTuple = BranchTuple.loadFromJson(branchIndex.packedJson,
                                                   branchIndex.importHash,
                                                   branchIndex.importGroupHash)

            self._liveEditController.updateBranch(branchTuple)

            self._updateBranchKeyToIdMap()

            self._tupleObservable.notifyOfTupleUpdate(
                TupleSelector(
                    BranchLiveEditTuple.tupleType(), dict(
                        coordSetId=tupleAction.branchTuple.coordSetId,
                        key=tupleAction.branchTuple.key
                    )
                )
            )

        except Exception as e:
            logger.exception(e)

        finally:
            dbSession.close()

    def _updateBranchKeyToIdMap(self):
        self._tupleObservable.notifyOfTupleUpdate(
            TupleSelector(BranchKeyToIdMapTuple.tupleType(), {})
        )



"""Added Poly template name

Peek Plugin Database Migration Script

Revision ID: a86e0317e22a
Revises: ec97d31aebb0
Create Date: 2019-09-29 22:24:23.878249

"""

# revision identifiers, used by Alembic.
revision = 'a86e0317e22a'
down_revision = 'ec97d31aebb0'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('DispPolyline', sa.Column('targetEdgeTemplateId', sa.Integer(), nullable=True), schema='pl_diagram')
    op.add_column('DispPolyline', sa.Column('targetEdgeTemplateName', sa.String(), nullable=True), schema='pl_diagram')
    op.create_index('idx_DispPolyline_targetEdgeTemplateId', 'DispPolyline', ['targetEdgeTemplateId'], unique=False, schema='pl_diagram')
    op.create_foreign_key("DispPolyline_targetEdgeTemplateId_fkey", 'DispPolyline', 'DispEdgeTemplate', ['targetEdgeTemplateId'], ['id'], source_schema='pl_diagram', referent_schema='pl_diagram', ondelete='SET NULL')
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint("DispPolyline_targetEdgeTemplateId_fkey", 'DispPolyline', schema='pl_diagram', type_='foreignkey')
    op.drop_index('idx_DispPolyline_targetEdgeTemplateId', table_name='DispPolyline', schema='pl_diagram')
    op.drop_column('DispPolyline', 'targetEdgeTemplateName', schema='pl_diagram')
    op.drop_column('DispPolyline', 'targetEdgeTemplateId', schema='pl_diagram')
    # ### end Alembic commands ###


"""Added branch fields to DispBase

Peek Plugin Database Migration Script

Revision ID: 4f6ed1047562
Revises: 361988cd327d
Create Date: 2019-05-11 15:05:14.096453

"""

# revision identifiers, used by Alembic.
revision = '4f6ed1047562'
down_revision = '361988cd327d'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('DispBase', sa.Column('branchId', sa.Integer(), nullable=True), schema='pl_diagram')
    op.add_column('DispBase', sa.Column('branchStage', sa.Integer(), nullable=True), schema='pl_diagram')
    op.add_column('DispBase', sa.Column('hashId', sa.String(), nullable=True), schema='pl_diagram')
    op.add_column('DispBase', sa.Column('replacesHashId', sa.String(), nullable=True), schema='pl_diagram')
    op.create_index('idx_Disp_branchId', 'DispBase', ['branchId'], unique=False, schema='pl_diagram')
    op.create_foreign_key('DispBase_branchId_fkey', 'DispBase', 'BranchIndex', ['branchId'], ['id'], source_schema='pl_diagram', referent_schema='pl_diagram', ondelete='CASCADE')
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint('DispBase_branchId_fkey', 'DispBase', schema='pl_diagram', type_='foreignkey')
    op.drop_index('idx_Disp_branchId', table_name='DispBase', schema='pl_diagram')
    op.drop_column('DispBase', 'replacesHashId', schema='pl_diagram')
    op.drop_column('DispBase', 'hashId', schema='pl_diagram')
    op.drop_column('DispBase', 'branchStage', schema='pl_diagram')
    op.drop_column('DispBase', 'branchId', schema='pl_diagram')
    # ### end Alembic commands ###


import logging
from typing import Union

from sqlalchemy.orm import joinedload
from twisted.internet.defer import Deferred

from peek_plugin_diagram._private.storage.ModelSet import ModelCoordSet
from vortex.DeferUtil import deferToThreadWrapWithLogger
from vortex.Payload import Payload
from vortex.TupleSelector import TupleSelector
from vortex.handler.TupleDataObservableHandler import TuplesProviderABC

logger = logging.getLogger(__name__)

class ServerCoordSetTupleProvider(TuplesProviderABC):
    def __init__(self, ormSessionCreator):
        self._ormSessionCreator = ormSessionCreator

    @deferToThreadWrapWithLogger(logger)
    def makeVortexMsg(self, filt: dict,
                      tupleSelector: TupleSelector) -> Union[Deferred, bytes]:

        session = self._ormSessionCreator()
        try:
            all = (session.query(ModelCoordSet)
                   .options(joinedload(ModelCoordSet.modelSet))
                   .all())

            for item in all:
                item.data = {"modelSetKey": item.modelSet.key}
                item.isLanding = item.modelSet.landingCoordSetId == item.id

            return Payload(filt, tuples=all).makePayloadEnvelope().toVortexMsg()

        finally:
            session.close()



import logging
from typing import List, Dict

import ujson
from peek_plugin_diagram._private.storage.ModelSet import ModelCoordSet

logger = logging.getLogger(__name__)


def _scaleDispGeomWithCoordSet(points: List[float],
                               coordSet: ModelCoordSet) -> List[float]:
    return _scaleDispGeom(points,
                          coordSet.multiplierX,
                          coordSet.multiplierY,
                          coordSet.offsetX,
                          coordSet.offsetY)


def _scaleDispGeom(points: List[float],
                   mx: float, my: float, ox: float, oy: float) -> List[float]:
    newGeom: List[float] = []

    for xi in range(0, len(points), 2):
        x = points[xi] * mx + ox
        y = points[xi + 1] * my + oy

        newGeom += [x, y]

    return newGeom


def _createHashId(dispDict: Dict) -> str:
    # Copy the input dict and pop the ID (we don't hash it)
    hashIdDict = dispDict.copy()

    # Delete the
    del hashIdDict['id']

    # Delete the branch id
    if 'bi' in hashIdDict: del hashIdDict['bi']

    # Delete the branch stage
    if 'bs' in hashIdDict: del hashIdDict['bs']

    # Delete the hash id
    if 'hid' in hashIdDict: del hashIdDict['hid']

    # Delete the replaces hash id
    if 'rid' in hashIdDict: del hashIdDict['rid']

    # Delete the group id
    if 'gi' in hashIdDict: del hashIdDict['gi']

    # Hash the actual content of the disp, and convert the integer to a string
    hashId = hash(ujson.dumps(hashIdDict, sort_keys=True))
    hashIdStr = __num_encode(hashId)

    # Return the hash
    return hashIdStr


import string

ALPHABET = string.ascii_uppercase + string.ascii_lowercase + \
           string.digits + '-_'
ALPHABET_REVERSE = dict((c, i) for (i, c) in enumerate(ALPHABET))
BASE = len(ALPHABET)
SIGN_CHARACTER = '$'


def __num_encode(n):
    if n < 0:
        return SIGN_CHARACTER + __num_encode(-n)
    s = []
    while True:
        n, r = divmod(n, BASE)
        s.append(ALPHABET[r])
        if n == 0: break
    return ''.join(reversed(s))


def __num_decode(s):
    if s[0] == SIGN_CHARACTER:
        return -__num_decode(s[1:])
    n = 0
    for c in s:
        n = n * BASE + ALPHABET_REVERSE[c]
    return n



"""Added polyline start/end types

Peek Plugin Database Migration Script

Revision ID: e8d6d02c0922
Revises: 6d3a96cf1955
Create Date: 2019-06-03 11:47:15.090106

"""

# revision identifiers, used by Alembic.
revision = 'e8d6d02c0922'
down_revision = '6d3a96cf1955'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('DispPolyline', sa.Column('endEndType', sa.Integer(), nullable=True), schema='pl_diagram')
    op.add_column('DispPolyline', sa.Column('startEndType', sa.Integer(), nullable=True), schema='pl_diagram')
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('DispPolyline', 'startEndType', schema='pl_diagram')
    op.drop_column('DispPolyline', 'endEndType', schema='pl_diagram')
    # ### end Alembic commands ###


import logging
from datetime import datetime
from typing import Dict, Optional, Tuple

import pytz
from peek_plugin_diagram._private.tuples.branch.BranchLiveEditTuple import \
    BranchLiveEditTuple
from peek_plugin_diagram._private.tuples.branch.BranchLiveEditTupleAction import \
    BranchLiveEditTupleAction
from peek_plugin_diagram._private.tuples.branch.BranchTuple import BranchTuple
from twisted.internet import task
from twisted.internet.defer import Deferred, inlineCallbacks
from vortex.DeferUtil import vortexLogFailure
from vortex.TupleAction import TupleActionABC
from vortex.TupleSelector import TupleSelector
from vortex.handler.TupleActionProcessor import TupleActionProcessorDelegateABC
from vortex.handler.TupleDataObservableHandler import TupleDataObservableHandler

logger = logging.getLogger(__name__)


class BranchLiveEditController(TupleActionProcessorDelegateABC):
    """ Branch Live Edit

    This controller just stores activly edited branches, and relays them between
    each other.

    """
    PERIOD = 30.000

    def __init__(self):
        self._tupleObservable = None

        #: This stores the cache of grid data for the clients
        self._cache: Dict[Tuple[str, int], BranchLiveEditTuple] = {}

        self._pollLoopingCall = task.LoopingCall(self._expireCheck)

    def start(self):
        d = self._pollLoopingCall.start(self.PERIOD, now=False)
        d.addErrback(vortexLogFailure, logger, consumeError=True)

    def _timerErrback(self, failure):
        vortexLogFailure(failure, logger)

    def _expireCheck(self, _):
        # Clean out the old branch updates
        for key, item in list(self._cache.items()):
            if (datetime.now(pytz.utc) - item.serverUpdateDate).days:
                self._cache.pop(key)

    def shutdown(self):
        if not self._pollLoopingCall:
            self._pollLoopingCall.stop()

        self._pollLoopingCall = None

        self._tupleObservable = None
        self._cache = {}

    def setTupleObservable(self, tupleObservable: TupleDataObservableHandler):
        self._tupleObservable = tupleObservable

    def updateBranch(self, branchTuple: BranchTuple):
        ts = TupleSelector(
            BranchLiveEditTuple.tupleType(), dict(
                coordSetId=branchTuple.coordSetId,
                key=branchTuple.key
            )
        )

        if not self._tupleObservable.hasTupleSubscribers(ts):
            return

        cacheKey = (branchTuple.coordSetId, branchTuple.key)

        liveEditTuple = BranchLiveEditTuple()
        liveEditTuple.branchTuple = branchTuple
        liveEditTuple.uiUpdateDate = branchTuple.updatedDate
        liveEditTuple.serverUpdateDate = datetime.now(pytz.utc)
        liveEditTuple.updateFromActionType = BranchLiveEditTuple.EDITING_SAVED
        self._cache[cacheKey] = liveEditTuple

    @inlineCallbacks
    def processTupleAction(self, tupleAction: TupleActionABC) -> Deferred:
        yield None

        assert isinstance(tupleAction, BranchLiveEditTupleAction), \
            "tupleAction is not BranchLiveEditTupleAction"

        # Ignore the updates
        if tupleAction.actionType == BranchLiveEditTupleAction.EDITING_STARTED:
            return

        coordSetId = tupleAction.branchTuple.coordSetId
        key = tupleAction.branchTuple.key

        tuple_ = BranchLiveEditTuple()
        tuple_.branchTuple = tupleAction.branchTuple
        tuple_.updatedByUser = tupleAction.updatedByUser
        tuple_.uiUpdateDate = tupleAction.dateTime
        tuple_.serverUpdateDate = datetime.now(pytz.utc)
        tuple_.updateFromActionType = tupleAction.actionType

        self._cache[(coordSetId, key)] = tuple_

        self._tupleObservable.notifyOfTupleUpdate(
            TupleSelector(BranchLiveEditTuple.tupleName(), {
                "coordSetId": coordSetId,
                "key": key
            })
        )

    def getLiveEditTuple(self, coordSetId, key) -> Optional[BranchLiveEditTuple]:
        return self._cache.get((coordSetId, key))



import logging
from typing import Dict

from peek_abstract_chunked_index.private.client.handlers.ACICacheHandlerABC import \
    ACICacheHandlerABC
from peek_abstract_chunked_index.private.tuples.ACIUpdateDateTupleABC import \
    ACIUpdateDateTupleABC
from peek_plugin_diagram._private.PluginNames import diagramFilt
from peek_plugin_diagram._private.tuples.branch.BranchIndexUpdateDateTuple import \
    BranchIndexUpdateDateTuple

logger = logging.getLogger(__name__)

clientBranchIndexWatchUpdateFromDeviceFilt = {
    'key': "clientBranchIndexWatchUpdateFromDevice"
}
clientBranchIndexWatchUpdateFromDeviceFilt.update(diagramFilt)


# ModelSet HANDLER
class BranchIndexCacheHandler(ACICacheHandlerABC):
    _UpdateDateTuple: ACIUpdateDateTupleABC = BranchIndexUpdateDateTuple
    _updateFromServerFilt: Dict = clientBranchIndexWatchUpdateFromDeviceFilt
    _logger: logging.Logger = logger



"""Initial

Peek Plugin Database Migration Script

Revision ID: 7eed363f5df5
Revises: 
Create Date: 2017-07-13 23:07:17.930374

"""

# revision identifiers, used by Alembic.
from sqlalchemy import func
from sqlalchemy.dialects.mssql.base import MSDialect
from sqlalchemy.dialects.postgresql.base import PGDialect

revision = '7eed363f5df5'
down_revision = None
branch_labels = None
depends_on = None

from sqlalchemy.schema import Sequence, CreateSequence, DropSequence

from alembic import op
import sqlalchemy as sa
import geoalchemy2



def isMssqlDialect():
    return isinstance(op.get_bind().engine.dialect, MSDialect)

def isPostGreSQLDialect():
    return isinstance(op.get_bind().engine.dialect, PGDialect)

def nextval(seqName):
    if isMssqlDialect():
        return sa.text('(NEXT VALUE FOR "pl_diagram"."%s")' % seqName)

    return sa.text('nextval(\'pl_diagram."%s"\')' % seqName)

def upgrade():

    op.execute(CreateSequence(Sequence('DispBase_id_seq',  minvalue=0, schema='pl_diagram')))
    op.execute(CreateSequence(Sequence('LiveDbDispLink_id_seq', minvalue=0,  schema='pl_diagram')))

    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('DispCompilerQueue',
    sa.Column('id', sa.Integer(), autoincrement=True, nullable=False),
    sa.Column('dispId', sa.Integer(), nullable=False),
    sa.PrimaryKeyConstraint('id', 'dispId'),
    schema='pl_diagram'
    )
    op.create_table('ModelSet',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('name', sa.String(length=50), nullable=False),
    sa.Column('comment', sa.String(), nullable=True),
    sa.PrimaryKeyConstraint('id'),
    schema='pl_diagram'
    )
    op.create_index('idx_ModelSet_name', 'ModelSet', ['name'], unique=True, schema='pl_diagram')
    op.create_table('Setting',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('name', sa.String(length=50), nullable=True),
    sa.PrimaryKeyConstraint('id'),
    schema='pl_diagram'
    )
    op.create_table('DispColor',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('name', sa.String(length=50), nullable=False),
    sa.Column('color', sa.String(length=20), server_default='orange', nullable=True),
    sa.Column('altColor', sa.String(length=20), nullable=True),
    sa.Column('swapPeriod', sa.Float(), nullable=True),
    sa.Column('modelSetId', sa.Integer(), nullable=False),
    sa.Column('importHash', sa.String(length=100), nullable=True),
    sa.ForeignKeyConstraint(['modelSetId'], ['pl_diagram.ModelSet.id'], ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('id'),
    schema='pl_diagram'
    )
    op.create_index('idx_DispColor_importHash', 'DispColor', ['importHash'], unique=True, schema='pl_diagram')
    op.create_index('idx_DispColor_modelSetId', 'DispColor', ['modelSetId'], unique=False, schema='pl_diagram')
    op.create_table('DispLayer',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('name', sa.String(length=50), nullable=False),
    sa.Column('order', sa.Integer(), server_default='0', nullable=False),
    sa.Column('selectable', sa.Boolean(), server_default='false', nullable=False),
    sa.Column('visible', sa.Boolean(), server_default='true', nullable=False),
    sa.Column('modelSetId', sa.Integer(), nullable=False),
    sa.Column('importHash', sa.String(length=100), nullable=True),
    sa.ForeignKeyConstraint(['modelSetId'], ['pl_diagram.ModelSet.id'], ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('id'),
    schema='pl_diagram'
    )
    op.create_index('idx_DispLayer_modelSetId', 'DispLayer', ['modelSetId'], unique=False, schema='pl_diagram')
    op.create_table('DispLineStyle',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('name', sa.String(length=50), nullable=False),
    sa.Column('backgroundFillDashSpace', sa.Boolean(), server_default='false', nullable=False),
    sa.Column('capStyle', sa.String(length=15), nullable=False),
    sa.Column('joinStyle', sa.String(length=15), nullable=False),
    sa.Column('dashPattern', sa.String(length=50), nullable=True),
    sa.Column('startArrowSize', sa.Integer(), nullable=True),
    sa.Column('endArrowSize', sa.Integer(), nullable=True),
    sa.Column('winStyle', sa.Integer(), nullable=False),
    sa.Column('modelSetId', sa.Integer(), nullable=False),
    sa.Column('importHash', sa.String(length=100), nullable=True),
    sa.ForeignKeyConstraint(['modelSetId'], ['pl_diagram.ModelSet.id'], ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('id'),
    schema='pl_diagram'
    )
    op.create_index('idx_DispLineStyle_importHash', 'DispLineStyle', ['importHash'], unique=True, schema='pl_diagram')
    op.create_index('idx_DispLineStyle_modelSetId', 'DispLineStyle', ['modelSetId'], unique=False, schema='pl_diagram')
    op.create_table('DispTextStyle',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('name', sa.String(length=50), nullable=False),
    sa.Column('fontName', sa.String(length=30), server_default='GillSans', nullable=False),
    sa.Column('fontSize', sa.Integer(), server_default='9', nullable=False),
    sa.Column('fontStyle', sa.String(length=30), nullable=True),
    sa.Column('scalable', sa.Boolean(), server_default='true', nullable=False),
    sa.Column('scaleFactor', sa.Integer(), server_default='1', nullable=False),
    sa.Column('modelSetId', sa.Integer(), nullable=False),
    sa.Column('importHash', sa.String(length=100), nullable=True),
    sa.ForeignKeyConstraint(['modelSetId'], ['pl_diagram.ModelSet.id'], ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('id'),
    schema='pl_diagram'
    )
    op.create_index('idx_DispTextStyle_importHash', 'DispTextStyle', ['importHash'], unique=True, schema='pl_diagram')
    op.create_index('idx_DispTextStyle_modelSetId', 'DispTextStyle', ['modelSetId'], unique=False, schema='pl_diagram')
    op.create_table('ModelConnType',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('name', sa.String(length=50), nullable=False),
    sa.Column('comment', sa.String(), nullable=True),
    sa.Column('modelSetId', sa.Integer(), nullable=False),
    sa.ForeignKeyConstraint(['modelSetId'], ['pl_diagram.ModelSet.id'], ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('id'),
    schema='pl_diagram'
    )
    op.create_index('idxConnTypeModelSetId', 'ModelConnType', ['modelSetId'], unique=False, schema='pl_diagram')
    op.create_table('ModelCoordSet',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('name', sa.String(length=50), nullable=False),
    sa.Column('initialPanX', sa.Float(), server_default='0', nullable=False),
    sa.Column('initialPanY', sa.Float(), server_default='0', nullable=False),
    sa.Column('initialZoom', sa.Float(), server_default='0', nullable=False),
    sa.Column('enabled', sa.Boolean(), server_default='false', nullable=False),
    sa.Column('comment', sa.String(), nullable=True),
    sa.Column('importId1', sa.Integer(), nullable=True),
    sa.Column('importId2', sa.String(length=100), nullable=True),
    sa.Column('modelSetId', sa.Integer(), nullable=False),
    sa.ForeignKeyConstraint(['modelSetId'], ['pl_diagram.ModelSet.id'], ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('id'),
    schema='pl_diagram'
    )
    op.create_index('idxCoordModelSetId', 'ModelCoordSet', ['modelSetId'], unique=False, schema='pl_diagram')
    op.create_index('idxCoordSetImportId1', 'ModelCoordSet', ['importId1'], unique=False, schema='pl_diagram')
    op.create_index('idxCoordSetImportId2', 'ModelCoordSet', ['importId2'], unique=False, schema='pl_diagram')
    op.create_index('idxCoordSetModelName', 'ModelCoordSet', ['modelSetId', 'name'], unique=True, schema='pl_diagram')
    op.create_table('ModelNodeType',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('name', sa.String(length=50), nullable=False),
    sa.Column('comment', sa.String(), nullable=True),
    sa.Column('modelSetId', sa.Integer(), nullable=False),
    sa.ForeignKeyConstraint(['modelSetId'], ['pl_diagram.ModelSet.id'], ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('id'),
    schema='pl_diagram'
    )
    op.create_index('idxNodeTypeModelSetId', 'ModelNodeType', ['modelSetId'], unique=False, schema='pl_diagram')
    op.create_table('SettingProperty',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('settingId', sa.Integer(), nullable=False),
    sa.Column('key', sa.String(length=50), nullable=False),
    sa.Column('type', sa.String(length=16), nullable=True),
    sa.Column('int_value', sa.Integer(), nullable=True),
    sa.Column('char_value', sa.String(length=7500), nullable=True),
    sa.Column('boolean_value', sa.Boolean(), nullable=True),
    sa.ForeignKeyConstraint(['settingId'], ['pl_diagram.Setting.id'], ),
    sa.PrimaryKeyConstraint('id'),
    schema='pl_diagram'
    )
    op.create_index('idx_SettingProperty_settingId', 'SettingProperty', ['settingId'], unique=False, schema='pl_diagram')
    op.create_table('DispLevel',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('name', sa.String(length=50), nullable=False),
    sa.Column('order', sa.Integer(), server_default='0', nullable=False),
    sa.Column('minZoom', sa.Float(), nullable=True),
    sa.Column('maxZoom', sa.Float(), nullable=True),
    sa.Column('coordSetId', sa.Integer(), nullable=False),
    sa.Column('importHash', sa.String(length=100), nullable=True),
    sa.ForeignKeyConstraint(['coordSetId'], ['pl_diagram.ModelCoordSet.id'], ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('id'),
    schema='pl_diagram'
    )
    op.create_index('idx_DispLevel_coordSetId', 'DispLevel', ['coordSetId'], unique=False, schema='pl_diagram')
    op.create_table('GridKeyCompilerQueue',
    sa.Column('id', sa.Integer(), nullable=False, autoincrement=True),
    sa.Column('gridKey', sa.String(length=30), nullable=False),
    sa.Column('coordSetId', sa.Integer(), nullable=False),
    sa.ForeignKeyConstraint(['coordSetId'], ['pl_diagram.ModelCoordSet.id'], ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('id', 'gridKey', 'coordSetId'),
    schema='pl_diagram'
    )
    op.create_index('idx_GKCompQueue_coordSetId_gridKey', 'GridKeyCompilerQueue', ['coordSetId', 'gridKey'], unique=False, schema='pl_diagram')
    op.create_table('GridKeyIndexCompiled',
    sa.Column('id', sa.Integer(), nullable=False, autoincrement=True),
    sa.Column('gridKey', sa.String(length=30), nullable=False),
    sa.Column('blobData', sa.LargeBinary(), nullable=False),
    sa.Column('lastUpdate', sa.String(length=50), nullable=False),
    sa.Column('coordSetId', sa.Integer(), nullable=False),
    sa.ForeignKeyConstraint(['coordSetId'], ['pl_diagram.ModelCoordSet.id'], ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('id'),
    schema='pl_diagram'
    )
    op.create_index('idx_GKIndexUpdate_coordSetId', 'GridKeyIndexCompiled', ['coordSetId'], unique=False, schema='pl_diagram')
    op.create_index('idx_GKIndexUpdate_gridKey', 'GridKeyIndexCompiled', ['gridKey'], unique=True, schema='pl_diagram')
    op.create_table('ModelNode',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('propsJson', sa.String(length=500), nullable=True),
    sa.Column('modelSetId', sa.Integer(), nullable=False),
    sa.Column('importId1', sa.Integer(), nullable=True),
    sa.Column('importId2', sa.String(length=100), nullable=True),
    sa.Column('typeId', sa.Integer(), nullable=False),
    sa.ForeignKeyConstraint(['modelSetId'], ['pl_diagram.ModelSet.id'], ondelete='CASCADE'),
    sa.ForeignKeyConstraint(['typeId'], ['pl_diagram.ModelNodeType.id']),
    sa.PrimaryKeyConstraint('id'),
    schema='pl_diagram'
    )
    op.create_index('idxNodeImportId1', 'ModelNode', ['importId1'], unique=False, schema='pl_diagram')
    op.create_index('idxNodeImportId2', 'ModelNode', ['importId2'], unique=False, schema='pl_diagram')
    op.create_index('idxNodeModelSetId', 'ModelNode', ['modelSetId'], unique=False, schema='pl_diagram')
    op.create_index('idxNodeTypeId', 'ModelNode', ['typeId'], unique=False, schema='pl_diagram')
    op.create_table('DispBase',
    sa.Column('id', sa.Integer(), server_default=nextval("DispBase_id_seq"), nullable=False),
    sa.Column('type', sa.Integer(), nullable=False),
    sa.Column('coordSetId', sa.Integer(), nullable=False),
    sa.Column('layerId', sa.Integer(), nullable=True),
    sa.Column('levelId', sa.Integer(), nullable=True),
    sa.Column('dispJson', sa.String(length=200000), nullable=True),
    sa.Column('importUpdateDate', sa.DateTime(), nullable=True),
    sa.Column('importHash', sa.String(length=100), nullable=True),
    sa.Column('importGroupHash', sa.String(length=100), nullable=True),
    sa.ForeignKeyConstraint(['coordSetId'], ['pl_diagram.ModelCoordSet.id'], ondelete='CASCADE'),
    sa.ForeignKeyConstraint(['layerId'], ['pl_diagram.DispLayer.id']),
    sa.ForeignKeyConstraint(['levelId'], ['pl_diagram.DispLevel.id']),
    sa.PrimaryKeyConstraint('id'),
    schema='pl_diagram'
    )
    op.create_index('idx_Disp_coordSetId_', 'DispBase', ['coordSetId'], unique=False, schema='pl_diagram')
    op.create_index('idx_Disp_importGroupHash', 'DispBase', ['importGroupHash'], unique=False, schema='pl_diagram')
    op.create_index('idx_Disp_importUpdateDate', 'DispBase', ['importUpdateDate'], unique=False, schema='pl_diagram')
    op.create_index('idx_Disp_layerId', 'DispBase', ['layerId'], unique=False, schema='pl_diagram')
    op.create_index('idx_Disp_levelId', 'DispBase', ['levelId'], unique=False, schema='pl_diagram')
    op.create_table('ModelConn',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('propsJson', sa.String(length=500), nullable=True),
    sa.Column('modelSetId', sa.Integer(), nullable=False),
    sa.Column('importId1', sa.Integer(), nullable=True),
    sa.Column('importId2', sa.String(length=100), nullable=True),
    sa.Column('typeId', sa.Integer(), nullable=False),
    sa.Column('srcId', sa.Integer(), nullable=False),
    sa.Column('dstId', sa.Integer(), nullable=False),
    sa.ForeignKeyConstraint(['dstId'], ['pl_diagram.ModelNode.id']),
    sa.ForeignKeyConstraint(['modelSetId'], ['pl_diagram.ModelSet.id'], ondelete='CASCADE'),
    sa.ForeignKeyConstraint(['srcId'], ['pl_diagram.ModelNode.id']),
    sa.ForeignKeyConstraint(['typeId'], ['pl_diagram.ModelConnType.id']),
    sa.PrimaryKeyConstraint('id'),
    schema='pl_diagram'
    )
    op.create_index('idxConnImportId1', 'ModelConn', ['importId1'], unique=False, schema='pl_diagram')
    op.create_index('idxConnImportId2', 'ModelConn', ['importId2'], unique=False, schema='pl_diagram')
    op.create_index('idxConnModelDstId', 'ModelConn', ['dstId'], unique=False, schema='pl_diagram')
    op.create_index('idxConnModelSetId', 'ModelConn', ['modelSetId'], unique=False, schema='pl_diagram')
    op.create_index('idxConnModelSrcId', 'ModelConn', ['srcId'], unique=False, schema='pl_diagram')
    op.create_index('idxConnModelTypeId', 'ModelConn', ['typeId'], unique=False, schema='pl_diagram')
    op.create_index('idxConnSrcDst', 'ModelConn', ['srcId', 'dstId'], unique=True, schema='pl_diagram')
    op.create_table('DispEllipse',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('xRadius', sa.Float(), server_default='10.0', nullable=False),
    sa.Column('yRadius', sa.Float(), server_default='10.0', nullable=False),
    sa.Column('rotation', sa.Float(), server_default='0', nullable=False),
    sa.Column('startAngle', sa.Float(), server_default='0', nullable=False),
    sa.Column('endAngle', sa.Float(), server_default='360', nullable=False),
    sa.Column('lineWidth', sa.Integer(), server_default='2', nullable=False),
    sa.Column('geomJson', sa.String(length=2000), nullable=False),
    sa.Column('fillColorId', sa.Integer(), nullable=True),
    sa.Column('lineColorId', sa.Integer(), nullable=True),
    sa.Column('lineStyleId', sa.Integer(), nullable=True),
    sa.ForeignKeyConstraint(['fillColorId'], ['pl_diagram.DispColor.id']),
    sa.ForeignKeyConstraint(['id'], ['pl_diagram.DispBase.id'], ondelete='CASCADE'),
    sa.ForeignKeyConstraint(['lineColorId'], ['pl_diagram.DispColor.id']),
    sa.ForeignKeyConstraint(['lineStyleId'], ['pl_diagram.DispLineStyle.id']),
    sa.PrimaryKeyConstraint('id'),
    schema='pl_diagram'
    )
    op.create_index('idx_DispEllipse_fillColorId', 'DispEllipse', ['fillColorId'], unique=False, schema='pl_diagram')
    op.create_index('idx_DispEllipse_lineColorId', 'DispEllipse', ['lineColorId'], unique=False, schema='pl_diagram')
    op.create_index('idx_DispEllipse_lineStyleId', 'DispEllipse', ['lineStyleId'], unique=False, schema='pl_diagram')
    op.create_table('DispGroup',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('name', sa.String(length=50), nullable=False),
    sa.ForeignKeyConstraint(['id'], ['pl_diagram.DispBase.id'], ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('id'),
    sa.UniqueConstraint('name'),
    schema='pl_diagram'
    )
    op.create_table('DispPolygon',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('cornerRadius', sa.Float(), server_default='0', nullable=False),
    sa.Column('lineWidth', sa.Integer(), server_default='2', nullable=False),
    sa.Column('geomJson', sa.String(length=200000), nullable=False),
    sa.Column('fillColorId', sa.Integer(), nullable=True),
    sa.Column('fillDirection', sa.Integer(), nullable=True),
    sa.Column('fillPercent', sa.Float(), nullable=True),
    sa.Column('lineColorId', sa.Integer(), nullable=True),
    sa.Column('lineStyleId', sa.Integer(), nullable=True),
    sa.ForeignKeyConstraint(['fillColorId'], ['pl_diagram.DispColor.id']),
    sa.ForeignKeyConstraint(['id'], ['pl_diagram.DispBase.id'], ondelete='CASCADE'),
    sa.ForeignKeyConstraint(['lineColorId'], ['pl_diagram.DispColor.id']),
    sa.ForeignKeyConstraint(['lineStyleId'], ['pl_diagram.DispLineStyle.id']),
    sa.PrimaryKeyConstraint('id'),
    schema='pl_diagram'
    )
    op.create_index('idx_DispPolygon_fillColorId', 'DispPolygon', ['fillColorId'], unique=False, schema='pl_diagram')
    op.create_index('idx_DispPolygon_lineColorId', 'DispPolygon', ['lineColorId'], unique=False, schema='pl_diagram')
    op.create_index('idx_DispPolygon_lineStyleId', 'DispPolygon', ['lineStyleId'], unique=False, schema='pl_diagram')
    op.create_table('DispPolyline',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('lineWidth', sa.Integer(), server_default='2', nullable=False),
    sa.Column('geomJson', sa.String(length=200000), nullable=False),
    sa.Column('lineColorId', sa.Integer(), nullable=True),
    sa.Column('lineStyleId', sa.Integer(), nullable=True),
    sa.ForeignKeyConstraint(['id'], ['pl_diagram.DispBase.id'], ondelete='CASCADE'),
    sa.ForeignKeyConstraint(['lineColorId'], ['pl_diagram.DispColor.id']),
    sa.ForeignKeyConstraint(['lineStyleId'], ['pl_diagram.DispLineStyle.id']),
    sa.PrimaryKeyConstraint('id'),
    schema='pl_diagram'
    )
    op.create_index('idx_DispPolyline_lineColorId', 'DispPolyline', ['lineColorId'], unique=False, schema='pl_diagram')
    op.create_index('idx_DispPolyline_lineStyleId', 'DispPolyline', ['lineStyleId'], unique=False, schema='pl_diagram')
    op.create_table('DispText',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('verticalAlign', sa.Integer(), server_default='-1', nullable=False),
    sa.Column('horizontalAlign', sa.Integer(), server_default='0', nullable=False),
    sa.Column('rotation', sa.Float(), server_default='0', nullable=False),
    sa.Column('text', sa.String(length=1000), server_default='new text label', nullable=False),
    sa.Column('textFormat', sa.String(length=1000), nullable=True),
    sa.Column('geomJson', sa.String(length=2000), nullable=False),
    sa.Column('colorId', sa.Integer(), nullable=True),
    sa.Column('textStyleId', sa.Integer(), nullable=True),
    sa.ForeignKeyConstraint(['colorId'], ['pl_diagram.DispColor.id']),
    sa.ForeignKeyConstraint(['id'], ['pl_diagram.DispBase.id'], ondelete='CASCADE'),
    sa.ForeignKeyConstraint(['textStyleId'], ['pl_diagram.DispTextStyle.id']),
    sa.PrimaryKeyConstraint('id'),
    schema='pl_diagram'
    )
    op.create_index('idx_DispText_colorId', 'DispText', ['colorId'], unique=False, schema='pl_diagram')
    op.create_index('idx_DispText_styleId', 'DispText', ['textStyleId'], unique=False, schema='pl_diagram')
    op.create_table('GridKeyIndex',
    sa.Column('gridKey', sa.String(length=30), nullable=False),
    sa.Column('dispId', sa.Integer(), nullable=False),
    sa.Column('coordSetId', sa.Integer(), nullable=False),
    sa.ForeignKeyConstraint(['coordSetId'], ['pl_diagram.ModelCoordSet.id']),
    sa.ForeignKeyConstraint(['dispId'], ['pl_diagram.DispBase.id'], ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('gridKey', 'dispId'),
    schema='pl_diagram'
    )
    op.create_index('idx_GridKeyIndex_coordSetId', 'GridKeyIndex', ['coordSetId'], unique=False, schema='pl_diagram')
    op.create_index('idx_GridKeyIndex_dispId', 'GridKeyIndex', ['dispId'], unique=False, schema='pl_diagram')
    op.create_index('idx_GridKeyIndex_gridKey', 'GridKeyIndex', ['gridKey'], unique=False, schema='pl_diagram')
    op.create_table('LiveDbDispLink',
    sa.Column('id', sa.Integer(), server_default=nextval("LiveDbDispLink_id_seq"), nullable=False),
    sa.Column('coordSetId', sa.Integer(), nullable=False),
    sa.Column('dispId', sa.Integer(), nullable=False),
    sa.Column('dispAttrName', sa.String(length=20), nullable=False),
    sa.Column('liveDbKey', sa.String(length=30), nullable=False),
    sa.Column('importKeyHash', sa.String(length=100), nullable=True),
    sa.Column('importGroupHash', sa.String(length=100), nullable=True),
    sa.Column('importDispHash', sa.String(length=100), nullable=True),
    sa.Column('propsJson', sa.String(length=500), nullable=True),
    sa.ForeignKeyConstraint(['coordSetId'], ['pl_diagram.ModelCoordSet.id']),
    sa.ForeignKeyConstraint(['dispId'], ['pl_diagram.DispBase.id'], ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('id'),
    schema='pl_diagram'
    )
    op.create_index('idx_LiveDbDLink_DispKeyHash', 'LiveDbDispLink', ['importKeyHash', 'importDispHash', 'dispAttrName'], unique=False, schema='pl_diagram')
    op.create_index('idx_LiveDbDLink_coordSetId', 'LiveDbDispLink', ['coordSetId'], unique=False, schema='pl_diagram')
    op.create_index('idx_LiveDbDLink_dispId', 'LiveDbDispLink', ['dispId'], unique=False, schema='pl_diagram')
    op.create_index('idx_LiveDbDLink_dispId_attr', 'LiveDbDispLink', ['dispId', 'dispAttrName'], unique=True, schema='pl_diagram')
    op.create_index('idx_LiveDbDLink_importGroupHash', 'LiveDbDispLink', ['importGroupHash'], unique=False, schema='pl_diagram')
    op.create_index('idx_LiveDbDLink_liveDbUpdate', 'LiveDbDispLink', ['dispId', 'liveDbKey'], unique=False, schema='pl_diagram')
    op.create_index('idx_LiveDbDLink_liveKeyId', 'LiveDbDispLink', ['liveDbKey'], unique=False, schema='pl_diagram')
    op.create_table('DispAction',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('propsJson', sa.String(length=500), nullable=True),
    sa.ForeignKeyConstraint(['id'], ['pl_diagram.DispPolygon.id'], ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('id'),
    schema='pl_diagram'
    )
    op.create_table('DispGroupItem',
    sa.Column('groupId', sa.Integer(), nullable=False),
    sa.Column('itemId', sa.Integer(), nullable=False),
    sa.ForeignKeyConstraint(['groupId'], ['pl_diagram.DispGroup.id'], ondelete='CASCADE'),
    sa.ForeignKeyConstraint(['itemId'], ['pl_diagram.DispBase.id']),
    sa.PrimaryKeyConstraint('groupId', 'itemId'),
    schema='pl_diagram'
    )
    op.create_table('DispGroupPointer',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('rotation', sa.Integer(), server_default='0', nullable=False),
    sa.Column('verticalScale', sa.Float(), server_default='1.0', nullable=False),
    sa.Column('horizontalScale', sa.Float(), server_default='1.0', nullable=False),
    sa.Column('geomJson', sa.String(length=2000), nullable=False),
    sa.Column('groupId', sa.Integer(), nullable=False),
    sa.ForeignKeyConstraint(['groupId'], ['pl_diagram.DispGroup.id']),
    sa.ForeignKeyConstraint(['id'], ['pl_diagram.DispBase.id'], ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('id'),
    schema='pl_diagram'
    )
    op.create_index('idxDispGroupPointer_groupId', 'DispGroupPointer', ['groupId'], unique=False, schema='pl_diagram')
    op.create_table('DispPolylineConn',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('connId', sa.Integer(), nullable=False),
    sa.Column('propsJson', sa.String(length=500), nullable=True),
    sa.ForeignKeyConstraint(['connId'], ['pl_diagram.ModelConn.id']),
    sa.ForeignKeyConstraint(['id'], ['pl_diagram.DispPolyline.id'], ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('id'),
    schema='pl_diagram'
    )
    op.create_index('idxConnCoordConnId', 'DispPolylineConn', ['connId'], unique=False, schema='pl_diagram')
    op.create_table('DispGroupPointerNode',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('nodeId', sa.Integer(), nullable=False),
    sa.ForeignKeyConstraint(['id'], ['pl_diagram.DispGroupPointer.id'], ondelete='CASCADE'),
    sa.ForeignKeyConstraint(['nodeId'], ['pl_diagram.ModelNode.id']),
    sa.PrimaryKeyConstraint('id'),
    schema='pl_diagram'
    )
    op.create_index('idxNodeCoordNodeId', 'DispGroupPointerNode', ['nodeId'], unique=False, schema='pl_diagram')
    # ### end Alembic commands ###


def downgrade():
    raise NotImplementedError()


"""Removed BranchGridIndex

Peek Plugin Database Migration Script

Revision ID: bf8f559c4df9
Revises: 4f6ed1047562
Create Date: 2019-05-11 15:55:51.316969

"""

# revision identifiers, used by Alembic.
revision = 'bf8f559c4df9'
down_revision = '4f6ed1047562'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index('idx_BranchGridIndex_coord_key', table_name='BranchGridIndex', schema='pl_diagram')
    op.drop_table('BranchGridIndex', schema='pl_diagram')
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('BranchGridIndex',
    sa.Column('id', sa.INTEGER(), server_default=sa.text('nextval(\'pl_diagram."BranchGridIndex_id_seq"\'::regclass)'), autoincrement=True, nullable=False),
    sa.Column('branchIndexId', sa.INTEGER(), autoincrement=False, nullable=False),
    sa.Column('coordSetId', sa.INTEGER(), autoincrement=False, nullable=False),
    sa.Column('gridKey', sa.VARCHAR(), autoincrement=False, nullable=False),
    sa.ForeignKeyConstraint(['branchIndexId'], ['pl_diagram.BranchIndex.id'], name='BranchGridIndex_branchIndexId_fkey', ondelete='CASCADE'),
    sa.ForeignKeyConstraint(['coordSetId'], ['pl_diagram.ModelCoordSet.id'], name='BranchGridIndex_coordSetId_fkey', ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('id', 'coordSetId', name='BranchGridIndex_pkey'),
    schema='pl_diagram'
    )
    op.create_index('idx_BranchGridIndex_coord_key', 'BranchGridIndex', ['coordSetId', 'branchIndexId', 'gridKey'], unique=True, schema='pl_diagram')
    # ### end Alembic commands ###


import logging

from peek_abstract_chunked_index.private.client.controller.ACICacheControllerABC import \
    ACICacheControllerABC
from peek_plugin_diagram._private.PluginNames import diagramFilt
from peek_plugin_diagram._private.server.client_handlers.ClientLocationIndexLoaderRpc import \
    ClientLocationIndexLoaderRpc
from peek_plugin_diagram._private.tuples.location_index.EncodedLocationIndexTuple import \
    EncodedLocationIndexTuple

logger = logging.getLogger(__name__)

clientLocationIndexUpdateFromServerFilt = dict(key="clientLocationIndexUpdateFromServer")
clientLocationIndexUpdateFromServerFilt.update(diagramFilt)


class LocationIndexCacheController(ACICacheControllerABC):
    """ Disp Key Cache Controller

    The encodedChunk cache controller stores all the locationIndexs in memory, allowing fast access from
    the mobile and desktop devices.

    """

    _ChunkedTuple = EncodedLocationIndexTuple
    _chunkLoadRpcMethod = ClientLocationIndexLoaderRpc.loadLocationIndexes
    _updateFromServerFilt = clientLocationIndexUpdateFromServerFilt
    _logger = logger



import logging

from peek_abstract_chunked_index.private.client.controller.ACICacheControllerABC import \
    ACICacheControllerABC
from peek_plugin_diagram._private.PluginNames import diagramFilt
from peek_plugin_diagram._private.server.client_handlers.BranchIndexChunkLoadRpc import \
    BranchIndexChunkLoadRpc
from peek_plugin_diagram._private.storage.branch.BranchIndexEncodedChunk import \
    BranchIndexEncodedChunk

logger = logging.getLogger(__name__)

clientBranchIndexUpdateFromServerFilt = dict(key="clientBranchIndexUpdateFromServer")
clientBranchIndexUpdateFromServerFilt.update(diagramFilt)


class BranchIndexCacheController(ACICacheControllerABC):
    """ BranchIndex Cache Controller

    The BranchIndex cache controller stores all the chunks in memory,
    allowing fast access from the mobile and desktop devices.

    """

    _ChunkedTuple = BranchIndexEncodedChunk
    _chunkLoadRpcMethod = BranchIndexChunkLoadRpc.loadBranchIndexChunks
    _updateFromServerFilt = clientBranchIndexUpdateFromServerFilt
    _logger = logger
